"""Curenttly, to work with flamewok, you may need to import some of
this objects:

Menu
TextBox
ActionBox
Form
Field
"""

from flamewok.menu import Menu, TextBox, ActionBox
from flamewok.form import Form, Field
from flamewok.validators import check_type
from flamewok.helpers import clear

__version__ = "1.0.5"
