
"""Here are defined the default values used in flamewok"""


# Menu default settings

DEFAULT_INSIDE_SEP = ": "
DEFAULT_SEP = " | "
DEFAULT_BOX_SIZE = 20
DEFAULT_LINE_LENGTH = 80
DEFAULT_MENU_PROMPT = "?> "
DEFAULT_MENU_ERROR = "\n!! Wrong entry, you must enter a valid choice !!"
DEFAULT_FIELD_ERROR = "Unexpected value, please try again"


# Form default settings

DEFAULT_FORM_PROMPT = "> "
